/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvdefaultgui.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <QVApplication>
#include <QVImageCanvas>

#include <qvgui/qvdefaultgui.h>
#include <qvgui/qvparamsinspectorwidget.h>
#include <qvgui/qvcamerawidgetsmall.h>
#include <qvgui/qvworkerinterfacesmall.h>

QVDefaultGUI::QVDefaultGUI(QWidget *parent): QWidget(parent), qvimagecanvaslist()
	{
	qDebug() << "QVDefaultGUI::QVDefaultGUI()";
	if (qvApp == NULL)
		{
		QString str =	"QVDefaultGUI::QVDefaultGUI(): the QVDefaultGUI cannot be created before the QVApplication instance. Aborting now.";
		std::cerr << qPrintable(str) << std::endl;
		exit(1);
		}
	// if its a --help call, do nothing
	if (qvApp->forHelp()) return;

	connect(this,SIGNAL(closed()),qvApp,SLOT(quit()));

	hboxlayout = new QHBoxLayout();
	leftwidget = new QWidget();
	qgroupbox = new QGroupBox("Workers input");
	leftvboxlayout = new QVBoxLayout(leftwidget);
	toolbox = new QToolBox(this);

	setWindowTitle("QVision default GUI");
	qvApp->registerGUI(this);

	qDebug() << "QVDefaultGUI::QVDefaultGUI() <~ return";
	}

QVDefaultGUI::~QVDefaultGUI()
	{
	QListIterator<QVImageCanvas*> i(qvimagecanvaslist);
	while (i.hasNext())
	delete(i.next());
	}

void QVDefaultGUI::init()
	{
	foreach(QVPropertyContainer* qvp, qvApp->getQVPropertyContainers())
		{
		QVWorker* worker;
		if((worker = dynamic_cast<QVWorker*>(qvp)) != NULL)
			add(worker);
		}
	leftvboxlayout->addStretch();
	foreach(QVPropertyContainer* qvp, qvApp->getQVPropertyContainers())
		{
		QVCamera* camera;
		if((camera = dynamic_cast<QVCamera*>(qvp)) != NULL)
			add(camera);
		}

	foreach(QVPropertyContainer* qvp, qvApp->getQVPropertyContainers())
		{
		QVImageCanvas *imageCanvas;
		if((imageCanvas = dynamic_cast<QVImageCanvas*>(qvp)) != NULL)
			add(imageCanvas);
		}

	leftvboxlayout->setSpacing(0);
	leftvboxlayout->setMargin(0);
	hboxlayout->addWidget(leftwidget);
	groupboxlayout = new QVBoxLayout(qgroupbox);
	groupboxlayout->addWidget(toolbox);
	hboxlayout->addWidget(qgroupbox);
	setLayout(hboxlayout);
	}

void QVDefaultGUI::closeEvent(QCloseEvent *event)
	{
	Q_UNUSED(event);
	qvApp->deregisterGUI();
	emit closed();
	}

void QVDefaultGUI::add(QVWorker * worker)
	{
	// Adding params
	QVParamsInspectorWidget *qvparamsinspectorwidget;
	qvparamsinspectorwidget = new QVParamsInspectorWidget(worker,this);
	toolbox->addItem(qvparamsinspectorwidget,worker->getName());
	qvparamsinspectorwidget->updateGeometry();

	// Adding worker interface small
	QVWorkerInterfaceSmall *workerWidget = new QVWorkerInterfaceSmall(worker);
	leftvboxlayout->addWidget(workerWidget);
	}

void QVDefaultGUI::add(QVCamera *camera)
	{
	QVCameraWidgetSmall *cameraWidget = new QVCameraWidgetSmall(camera);
	leftvboxlayout->addWidget(cameraWidget);
	}

void QVDefaultGUI::add(QVImageCanvas *imageCanvas)
	{
	// Adding params
	QVParamsInspectorWidget *qvparamsinspectorwidget;
	qvparamsinspectorwidget = new QVParamsInspectorWidget(imageCanvas,this);
	toolbox->addItem(qvparamsinspectorwidget, imageCanvas->getName());
	qvparamsinspectorwidget->updateGeometry();
	}

