/*
 *	Copyright (C) 2007, 2008. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvdefaultgui.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVDEFAULTGUI_H
#define QVDEFAULTGUI_H

#include <QWidget>
#include <QMap>
#include <QCloseEvent>
#include <QVGUI>

#include <QToolBox>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGroupBox>

/// @deprecated qvdefaultgui.h should not include qvimagecanvas.h
#include <QVImageCanvas>
#include <QVCamera>
#include <QVWorker>
#include <qvgui/qvplot.h>

class QVImageCanvas;

/*!
@class QVDefaultGUI qvdefaultgui.h qvdefaultgui/qvdefaultgui.h
@brief Main user interface widget for QVision applications.

This is the default graphical user interface class. It offers basic widgets to control several aspects of the execution of a QVision application:

@image html qvisioninterface_rotoscoper.png

For further information about this widget and its usage see section @ref QVDefaultGUIwidget.


@ingroup qvgui
*/
class QVDefaultGUI: public QWidget, QVGUI
{
	Q_OBJECT
public:
	//Ui::QVisionForm form;

	/// @brief Constructs a QVDefaultGUI object, given the QVision object which work (properties, result images,
	/// etc...) will be inspected.
	/// @param name identifier for the object to be created.
	QVDefaultGUI(QWidget *parent = 0);
	~QVDefaultGUI();

	void closeEvent(QCloseEvent *event);

	/// @brief initialice the QVDefaultGUI object, adding all workers and canvas registeres in the QVApplication
	/// etc...) will be inspected.
	void init();

	void show() { QWidget::show(); }

signals:
	/// @brief Emited when the image canvas window closes.
	void closed();

private:
	#ifndef DOXYGEN_IGNORE_THIS
	// Adds a new QVPropertyContainer, which double, integer and boolean properties will be added to the  interface to be
	// controlled in real time by the final user
	// the camera.
	// param name Identification name of the QVPropertyContainer in the interface
	// param object QVPropertyContainer object which property controls will be shown
	void add(QVWorker * worker);

	// Adds controls for a QVCamera object to the interface, allowing the user to control and inspect the state of
	// the camera.
	// param camera QVCamera object which will be controlled
	void add(QVCamera *camera);

	void add(QVImageCanvas *imageCanvas);

	//bool setVisible(QVWorker *worker,QString prop_orig,float low=0.0,float high=255.0);

	QHBoxLayout *hboxlayout;
	QWidget *leftwidget;
	QToolBox *toolbox;
	QGroupBox *qgroupbox;
	QVBoxLayout *leftvboxlayout;
	QVBoxLayout *groupboxlayout;
	QList<QVImageCanvas*> qvimagecanvaslist;
	#endif
};

#endif
